@@ -58,7 +58,24 @@ DEFAULT_ARTIFICIAL_END_ACTIVITY = "■"
 
 DEFAULT_BUSINESS_HOURS_WORKCALENDAR = None
 
+SHOW_EVENT_LOG_DEPRECATION = True
+TRIGGERED_DT_PARSING_WARNING = False
+
 DEFAULT_BGCOLOR = "white"
+DEFAULT_FORMAT_GVIZ_VIEW = "png"
+
+ENABLE_MULTIPROCESSING_DEFAULT = False
+DEFAULT_READ_XES_LEGACY_OBJECT = False
+
+# Default business hour slots: Mondays to Fridays, 7:00 - 17:00 (in seconds)
+DEFAULT_BUSINESS_HOUR_SLOTS = [
+    ((0 * 24 + 7) * 60 * 60, (0 * 24 + 17) * 60 * 60),
+    ((1 * 24 + 7) * 60 * 60, (1 * 24 + 17) * 60 * 60),
+    ((2 * 24 + 7) * 60 * 60, (2 * 24 + 17) * 60 * 60),
+    ((3 * 24 + 7) * 60 * 60, (3 * 24 + 17) * 60 * 60),
+    ((4 * 24 + 7) * 60 * 60, (4 * 24 + 17) * 60 * 60),
+]
+
 
 from enum import Enum
 
